import java.text.DecimalFormat;

/**
 * Draws a stripchart recorder.
 */
class Stripchart {
  int x;           // horizontal position of chart
  int y;           // vertical position of chart
  int nSamples;    // number of samples to display (affects width)
  int h;           // height of chart
  int period;      // how often to draw a gray line
  float minValue;  // minimum value to display
  float maxValue;  // maximum value to display
  Channel [] channels; // data channels
  private DecimalFormat d = new DecimalFormat("0.#");
  private String minString;  // minimum value as a string
  private String maxString;  // maximum value as a string
  
  private PFont legendFont = loadFont("ArialMT-10.vlw");

  /*
    rightSpace tells how much room there is for the chart
    legend. VSPACE and HSPACE give the spacing from the border
    of the stripchart to the point plotting area.
  */
  private int rightSpace = 0;
  final static int VSPACE = 2;
  final static int HSPACE = 2;

  Stripchart(int x, int y, int nSamples, int h, int period,
    float minValue, float maxValue, int nChannels)
  {
    this.x = x;
    this.y = y;
    this.nSamples = nSamples;
    this.h = h;
    this.period = period;
    // make sure minimum and max are in proper order
    this.minValue = min(minValue, maxValue);
    this.maxValue = max(minValue, maxValue);
    
    // and convert them to a string with minimal numbr of decimal places
    this.minString = d.format(minValue);
    this.maxString = d.format(maxValue);
    channels = new Channel[nChannels];
  }
  
  Stripchart(int x, int y, int nSamples, int h, int period, color c,
  float minValue, float maxValue)
  {
    this(x, y, nSamples, h, period, minValue, maxValue, 1);
    this.channels[0] = new Channel(this, c);
  }

  Stripchart( int x, int y, int w, int h, int period, color c)
  {
    this(x, y, w, h, period, h/2.0, -h/2.0, 1);
    this.channels[0] = new Channel(this, c);
  }
  
  void setChannel(int cNum, color c)
  {
    if (cNum >= 0 && cNum < channels.length)
    {
      if (channels[cNum] == null)
      {
        channels[cNum] = new Channel(this, c);
      }
      else
      {
        channels[cNum].colour = c;
      }
    }
  } 

  /**
   * Add a data point to be plotted in the given channel
   *
   * @param cNum the channel number to add data to
   * @param value the value to plot
   */
  void addData(int cNum, float value)
  {
    value = constrain(value, minValue, maxValue);
    channels[cNum].addData(value);
  }
  
  /*
    Convenience method for strip charts with only one channel
  */
  void addData(float value)
  {
    addData(0, value);
  }
  
  void display()
  {
    stroke(0);
    fill(255);
    pushMatrix();
    translate(x, y);
    textFont(legendFont);
    
    // reserve space for the max/min value legend
    rightSpace = max(int(textWidth(minString)), int(textWidth(maxString)));
    rect(0, 0, nSamples + rightSpace + 2 * HSPACE, h + 2 * VSPACE);
    stroke(192);
    line(HSPACE, VSPACE + h / 2, nSamples + rightSpace - HSPACE, VSPACE + h / 2);
    line(nSamples + 1, VSPACE, nSamples + 1, h - VSPACE);
    
    String sub_1 = d.format((maxValue-minValue)/2);
    // draw max and min values
    textFont(legendFont);
    fill(0);
    stroke(0);
    text(minString, nSamples + 2, h - VSPACE);
    text(sub_1, nSamples + 2, (h/2) - VSPACE);    
    text(maxString, nSamples + 2, VSPACE + 8);
   
    for (int cNum = 0; cNum < channels.length; cNum++)
    {
      channels[cNum].display();
    }
    popMatrix();
  }

  /**
   * Add a data value and re-display the strip chart.
   * The addData() and display() methods are decoupled;
   * this lets you "speed up" the chart by adding
   * several points before displaying the chart.
   * This method is a convenience method that does
   * both actions.
   *
   *  @cNum the channel in which to plot data
   *  @param value the value to add and display
   */
  void plot(int cNum, float value)
  {
    addData(cNum, value);
    display();
  }
  
  /*
   * Convenience method for stripcharts with only one channel
   */
  void plot(float value)
  {
    plot(0, value);
  }
}


